import logging
from abc import ABC, abstractmethod

from xmlschema.validators.simple_types import *
from xmlschema.validators.complex_types import *
from xmlschema.validators.groups import *
from xmlschema.validators.facets import *

log = logging.getLogger()

class TypeMapping(ABC):
    ns_to_id_map = {}

    XSD_NS = "http://www.w3.org/2001/XMLSchema"

    XSD_TYPE_MAP = {
        "string" : { "type" : "string" },
        "normalizedString" : { "type" : "string"},
        "dateTime" : { "type" : "string"},
        "token" : { "type" : "string"},
        "anyURI" : { "type" : "string" },

        "integer" : { "type" : "integer"},
        "nonNegativeInteger" : { "type" : "integer", "minimum" : 0},
        "positiveInteger" : { "type" : "integer", "minimum" : 1},

        "boolean" : { "type" : "boolean" },

        "hexBinary" : { "type" : "string", "pattern" : "^([a-fA-F0-9]{2})*$"},
        "base64Binary" : { "type" : "string", "pattern" : "^[A-Za-z0-9+\/]*={0,3}$"},

        "anyType" : {}
    }    

    @abstractmethod
    def map(self, xst : BaseXsdType):
        return None

    @classmethod
    def extract_namespace(cls, qname: str):
        match = re.search(r'^\{([^\{\}]+)\}(([^\{\}]+))$', qname)
        if match is None:
            return None
        return match.group(1)

    @classmethod
    def get_ref_for(cls, xsd_type: XsdType, current_ns : str):
        ns = cls.extract_namespace(xsd_type.name)
        if ns == current_ns:
            return { "$ref" : f"#/$defs/{xsd_type.local_name}" }
        else:
            mapped_id = cls.ns_to_id_map[ns]
            return { "$ref" : f"{mapped_id['id']}#/$defs/{xsd_type.local_name}"}        

    @classmethod
    def get_type_from_elem(cls, elem: XsdElement, current_ns : str):
        ns = cls.extract_namespace(elem.type.name)
        if (ns == TypeMapping.XSD_NS):
            # this should be an XSD primitive type
            return dict(TypeMapping.XSD_TYPE_MAP[elem.type.local_name])
        else:
            return cls.get_ref_for(elem.type, current_ns)








